#include <stdlib.h>
#include <curses.h>
#include <time.h>

#include "tools.h"
#include "termset.h"
#include "direction.h"
#include "help.h"

#define CONT				0
#define QUIT				1

static char char_syms[20] = "!/#$%^&*(){[}];:<>?=";

/* Represents a character on the screen */
typedef struct _character
{
	int x;				/* x pos */
	int y;        /* y pos */
	int isKitten;	/* 1 if kitten */
	char sym;			/* symbol representing this character */
	int quotenum;	/* quote number assigned in quote database */
} Character;

int main(int argc, char* argv[])
{
	int done = CONT;
	int dir_mode = ARROW_MODE;
	int input;
	int count,check,redopl = 0;
	int moveguy = 1;
	int x,y,prevx,prevy;
	Direction direction;
	Character characters[20];
	char seeding_array[MAXARRAY];
	char quotearray[MAXARRAY][MAXARRAY];
	int numofquotes;

	/* Load the quote array, return error  if failed */
	if ((numofquotes = loadquotes (quotearray, MAXARRAY, MAXARRAY, 
																 "quotes.txt")) == -1)
		return 1;

	/* Check command line arguments */
	if (argc > 1)
	{
		for (count = 1; count < argc; count++)
		{
			if (argv[count][0] == '-')
			{
				switch (argv[count][1])
				{
					case 'v':
						dir_mode = VI_MODE;
						break;
					case 'h':
					default:
						cmdline_help();
						return 0;
						break;
				} /* END switch (argv[count][1]) */
			} /* END if (argv[count][0] == '-' */
			else /* if (argv[count][0] != '-') */
			{
				cmdline_help();
				return 0;
			} /* END else if (argv[count][0] != '-') */
		} /* END for (count = 1; count <= argc; count++) */
	} /* END if (argc > 1) */

	/* Set up the game terminal */
	set_gameterm();
	
	/* Reset the terminal whenever we exit */
	atexit(reset_gameterm);	

	/* Place the characters around the screen at random */
	initstate (time(NULL), seeding_array, 256);
	setstate (seeding_array);
	for (count = 0; count < 20; count++)
	{
		characters[count].isKitten = 0;
		characters[count].sym = char_syms[count];
		do
		{
			characters[count].x = random() % 80;
			characters[count].y = random() % 23 + 1;
			characters[count].quotenum = random() % numofquotes;
			for (check = 0; check < count; check++)
			{
				if (characters[count].x == characters[check].x
						&& characters[count].y == characters[check].y)
				{
					redopl = 1;
					break;
				}
				else if (characters[count].quotenum == characters[check].quotenum)
				{
					redopl = 1;
					break;
				}
				else
					redopl = 0;
			}
		} while (redopl == 1);
	} /* END for (count = 0; count < 20; count++) */

	/* Make one of the characters the kitten */
	characters[random() % 20].isKitten = 1;

	/* Place the characters randomly about the screen */
	for (count = 0; count < 20; count++)
	{
		move(characters[count].y,characters[count].x);
		addch(characters[count].sym);
	} /* END for (count = 0; count < 20; count++) */
	move(y = random() % 23 + 1, x = random() % 80);
	addch('@');

	/* Game Loop */
	while (!done)
	{
		/* Get input */
		input = getch ();

		prevx = x;
		prevy = y;

		direction = check_direction (input, dir_mode);
		switch (direction)
		{
			case dir_left:
				x--;
				break;
			case dir_down:
				y++;
				break;
			case dir_up:
				y--;
				break;
			case dir_right:
				x++;
				break;
			case dir_none:
			default:
				/* Check for none directional input */
				switch (input)
				{
					case '?':
						display_help();
						break;
					case 'q':
					case 'Q':
						done = QUIT;
						break;
					default:
						break;
				} /* END switch (input) */
		} /* END switch (direction) */
				
		for (count = 0; count < 20; count++)
		{
			if (x == characters[count].x && y == characters[count].y)
			{
				moveguy = 0;
				x = prevx;
				y = prevy;
			
				move(0,0);
				if (characters[count].isKitten)
				{
					addstr("You say, \"OH MY GOD! Mr. ");
					addch(characters[count].sym);
					addstr("! I had no idea YOU are the KITTEN!\"");
					done = QUIT;
				} /* END if (characters[count].isKitten) */
				else /* if (!characters[count].isKitten) */
				{
					addstr(quotearray[characters[count].quotenum]);
				} /* END else if (!characters[count].isKitten) */
				break;
			} /* END if (x == characters[count].x && y == characters[count].y) */
		} /* END for (count = 0; count < 20; count++) */
		if (moveguy)
		{
			move(prevy,prevx);
			addch(' ');
			if (x == 80) x = 0;
			if (y == 24) y = 1;
			if (x == -1) x = 79;
			if (y == -1) y = 22;
			move(y,x);
			addch('@');
			// clear line 1
			move(0,0);
			clrtoeol();
		} /* END if (moveguy) */
		else /* if (!moveguy) */
			moveguy = 1;
		
		refresh();
	} /* END Game Loop */

	return 0;
} /* END FUNC main */
